@@ -132,12 +132,16 @@ class Document(object):
 
         self.possible_headers_style = [x for x in reversed(sorted(_headers))]
 
+        _text_list = collections.Counter()
+
         for font_size, amount in six.iteritems(self.usage_font_size):
             if float(amount) / max_count <= 0.1:                
                 if font_size not in _headers:
                     _headers.append(font_size)
             else:
+                # This will require some cleanup
                 _text.append(font_size)
+                _text_list[font_size] = amount
 
         self.possible_headers = [x for x in reversed(sorted(_headers))]
         self.possible_text = [x for x in reversed(sorted(_text))]
@@ -149,12 +153,17 @@ class Document(object):
                     self.possible_headers.remove(value)
 #                    self.possible_headers_style.remove(value)
 
+        _mc = _text_list.most_common(1)
+
+        if len(_mc) > 0:
+            self.base_font_size = _mc[0][0]
 
     def reset(self):
         self.elements = []
         self.relationships = {}
         self.footnotes = {}
         self.endnotes = {}
+        self.comments = {}
         self.numbering = {}
         self.abstruct_numbering = {}
         self.styles = StylesCollection()
@@ -166,6 +175,16 @@ class Document(object):
         self.possible_headers_style = []
         self.possible_headers = []
         self.possible_text = []
+        self.base_font_size = -1
+
+
+class CommentContent:
+    def __init__(self, cid):
+        self.cid = cid
+        self.text = ''
+        self.elements = []
+        self.date = None
+        self.author = None
 
 
 class Element(object):
@@ -279,6 +298,19 @@ class Table(Element):
         return self.rows
 
 
+class Comment(Element):
+    "Represents comment element."
+
+    def __init__(self, cid, comment_type):
+        super(Comment, self).__init__()
+
+        self.cid = cid
+        self.comment_type = comment_type
+
+    def value(self):
+        return self.cid
+
+
 class Footnote(Element):
     "Represents footnote element."
 
@@ -450,3 +482,13 @@ class Math(Element):
     def value(self):
         return ''
 
+
+class SmartTag(Element):
+    "Represents SmartTag element."
+
+    def __init__(self):
+        self.elements = []
+        self.element = ''
+
+    def value(self):
+        return ''
