@@ -59,7 +59,7 @@ def parse_previous_properties(document, paragraph, prop):
 
         if isinstance(paragraph, doc.Text):
             if not ('dropcap' in paragraph.ppr  and paragraph.ppr['dropcap']):
-                if paragraph.parent and (not ('dropcap' in paragraph.parent.ppr  and paragraph.parent.ppr['dropcap'])):
+                if paragraph.parent and hasattr(paragraph.parent, 'ppr') and (not ('dropcap' in paragraph.parent.ppr  and paragraph.parent.ppr['dropcap'])):
                     document.add_font_as_used(paragraph.rpr['sz'])            
         elif isinstance(paragraph, doc.Paragraph):
             if not ('dropcap' in paragraph.ppr  and paragraph.ppr['dropcap']):
@@ -180,11 +180,14 @@ def parse_drawing(document, container, elem):
     We don't do much with drawing element. We can find embeded image but we don't do more than that.
     """
 
-    blip = elem.xpath('.//a:blip', namespaces=NAMESPACES)[0]        
-    _rid =  blip.attrib[_name('{{{r}}}embed')]
+    _blip = elem.xpath('.//a:blip', namespaces=NAMESPACES)
 
-    img = doc.Image(_rid)
-    container.elements.append(img)
+    if len(_blip) > 0:
+        blip = _blip[0]
+        _rid =  blip.attrib[_name('{{{r}}}embed')]
+
+        img = doc.Image(_rid)
+        container.elements.append(img)
 
 
 def parse_footnote(document, container, elem):
@@ -255,9 +258,7 @@ def parse_text(document, container, element):
         # Notice it is using txt as container
         parse_previous_properties(document, txt, rpr)
 
-    r = element.find(_name('{{{w}}}r'))
-
-    if r is not None:
+    for r in element.findall(_name('{{{w}}}r')):
         parse_text(document, container, r)
 
     foot = element.find(_name('{{{w}}}footnoteReference'))
@@ -283,6 +284,31 @@ def parse_text(document, container, element):
     if image is not None:
         parse_drawing(document, container, image)
 
+    refe = element.find(_name('{{{w}}}commentReference'))
+
+    if refe is not None:
+        _m = doc.Comment(refe.attrib[_name('{{{w}}}id')], 'reference')
+        container.elements.append(_m)
+
+    return
+
+
+def parse_smarttag(document, container, tag_elem):
+    "Parse the endnote element."
+
+    tag = doc.SmartTag()
+
+    tag.element = tag_elem.attrib[_name('{{{w}}}element')]
+
+    for elem in tag_elem:
+        if elem.tag == _name('{{{w}}}r'):
+            parse_text(document, tag, elem)      
+
+        if elem.tag == _name('{{{w}}}smartTag'):
+            parse_smarttag(document, tag, elem)            
+
+    container.elements.append(tag)
+
     return
 
 
@@ -310,6 +336,14 @@ def parse_paragraph(document, par):
             _m = doc.Math()
             paragraph.elements.append(_m)
 
+        if elem.tag == _name('{{{w}}}commentRangeStart'):
+            _m = doc.Comment(elem.attrib[_name('{{{w}}}id')], 'start')
+            paragraph.elements.append(_m)
+
+        if elem.tag == _name('{{{w}}}commentRangeEnd'):
+            _m = doc.Comment(elem.attrib[_name('{{{w}}}id')], 'end')
+            paragraph.elements.append(_m)
+
         if elem.tag == _name('{{{w}}}hyperlink'):
             try:
                 t = doc.Link(elem.attrib[_name('{{{r}}}id')])
@@ -320,6 +354,9 @@ def parse_paragraph(document, par):
             except:
                 logger.error('Error with with hyperlink [%s].', str(elem.attrib.items()))
 
+        if elem.tag == _name('{{{w}}}smartTag'):
+            parse_smarttag(document, paragraph, elem)            
+
     return paragraph
 
 
@@ -515,6 +552,30 @@ def parse_style(document, xmlcontent):
            parse_paragraph_properties(document, st, ppr)
 
 
+def parse_comments(document, xmlcontent):
+    """Parse comments document.
+
+    Comments are defined in file 'comments.xml'
+    """
+
+    comments = etree.fromstring(xmlcontent)
+    document.comments = {}
+
+    for comment in comments.xpath('.//w:comment', namespaces=NAMESPACES):
+        # w:author
+        # w:id
+        # w: date
+        comment_id = comment.attrib[_name('{{{w}}}id')]
+
+        comm = doc.CommentContent(comment_id)
+        comm.author = comment.attrib.get(_name('{{{w}}}author'), None)
+        comm.date = comment.attrib.get(_name('{{{w}}}date'), None)
+
+        comm.elements = [parse_paragraph(document, para) for para in comment.xpath('.//w:p', namespaces=NAMESPACES)]
+
+        document.comments[comment_id] = comm
+
+
 def parse_footnotes(document, xmlcontent):
     """Parse footnotes document.
 
@@ -531,7 +592,6 @@ def parse_footnotes(document, xmlcontent):
         if _type in ['separator', 'continuationSeparator', 'continuationNotice']:
             continue
 
-#        p = parse_paragraph(document, footnote.find(_name('{{{w}}}p')))
         paragraphs = [parse_paragraph(document, para) for para in footnote.xpath('.//w:p', namespaces=NAMESPACES)]
 
         document.footnotes[footnote.attrib[_name('{{{w}}}id')]] = paragraphs
@@ -614,6 +674,12 @@ def parse_from_file(file_object):
     except KeyError:
         logger.warning('Could not read relationships.')
 
+    try:    
+        comments_content = file_object.read_file('comments.xml')
+        parse_comments(document, comments_content)    
+    except KeyError:
+        logger.warning('Could not read footnotes.')
+
     try:    
         footnotes_content = file_object.read_file('footnotes.xml')
         parse_footnotes(document, footnotes_content)    
