@@ -31,6 +31,7 @@ Hooks:
 import os.path
 import six
 import collections
+import math
 
 from lxml import etree
 from . import doc
@@ -272,10 +273,26 @@ def serialize_link(ctx, document, elem, root):
 
     _a = etree.SubElement(root, 'a')
 
-    # this should not take just first element
-    if len(elem.elements) > 0:
-        _a.text = elem.elements[0].value()
-    _a.set('href', document.relationships[elem.rid]['target'])
+    for el in elem.elements:
+        _ser = ctx.get_serializer(el)
+
+        if _ser:
+            _td = _ser(ctx, document, el, _a)
+        else:
+            if isinstance(el, doc.Text):
+                children = list(_a)
+
+                if len(children) == 0:
+                    _text = _a.text or u''
+
+                    _a.text = u'{}{}'.format(_text, el.value())
+                else:
+                    _text = children[-1].tail or u''
+
+                    children[-1].tail = u'{}{}'.format(_text, el.value())
+
+    if elem.rid in document.relationships:
+        _a.set('href', document.relationships[elem.rid].get('target', ''))
 
     fire_hooks(ctx, document, elem, _a, ctx.get_hook('a'))
 
@@ -288,13 +305,14 @@ def serialize_image(ctx, document, elem, root):
     This is not abstract enough.
     """
 
-    img_src = document.relationships[elem.rid]['target']
-
-    img_name, img_extension = os.path.splitext(img_src)
-
     _img = etree.SubElement(root, 'img')
     # make path configurable
-    _img.set('src', 'static/{}{}'.format(elem.rid, img_extension))
+
+    if elem.rid in document.relationships:
+        img_src = document.relationships[elem.rid].get('target', '')
+        img_name, img_extension = os.path.splitext(img_src)
+
+        _img.set('src', 'static/{}{}'.format(elem.rid, img_extension))
 
     fire_hooks(ctx, document, elem, _img, ctx.get_hook('img'))
 
@@ -354,7 +372,7 @@ def get_style_fontsize(node):
     return 0
 
 
-def get_style_css(ctx, node, embed=True):
+def get_style_css(ctx, node, embed=True, fontsize=-1):
     """Returns as string defined CSS for this node.
 
     Defined CSS can be different if it is embeded or no. When it is embeded styling 
@@ -375,57 +393,60 @@ def get_style_css(ctx, node, embed=True):
     if not node:
         return
 
-    # temporarily
-    if not embed:
-        if 'b' in node.rpr:
-            style.append('font-weight: bold')
+    if fontsize in [-1, 2]:
+        if 'sz' in node.rpr:
+            size = int(node.rpr['sz']) / 2
 
-        if 'i' in node.rpr:
-            style.append('font-style: italic')
+            if ctx.options['embed_fontsize']:
+                if ctx.options['scale_to_size']:                
+                    multiplier = size-ctx.options['scale_to_size']
+                    scale = 100 + int(math.trunc(8.3*multiplier))
+                    style.append('font-size: {}%'.format(scale))
+                else:
+                    style.append('font-size: {}pt'.format(size))
 
-        if 'u' in node.rpr:
-            style.append('text-decoration: underline')
+    if fontsize in [-1, 1]:
+        # temporarily
+        if not embed:
+            if 'b' in node.rpr:
+                style.append('font-weight: bold')
 
-    if 'small_caps' in node.rpr:
-        style.append('font-variant: small-caps')
+            if 'i' in node.rpr:
+                style.append('font-style: italic')
 
-    if 'strike' in node.rpr:
-        style.append('text-decoration: line-through')
+            if 'u' in node.rpr:
+                style.append('text-decoration: underline')
 
-    if 'color' in node.rpr:
-        if node.rpr['color'] != '000000':
-            style.append('color: #{}'.format(node.rpr['color']))
+        if 'small_caps' in node.rpr:
+            style.append('font-variant: small-caps')
 
-    if 'jc' in node.ppr:
-        # left right both
-        align = node.ppr['jc']
-        if align.lower() == 'both':
-            align = 'justify'
+        if 'strike' in node.rpr:
+            style.append('text-decoration: line-through')
 
-        style.append('text-align: {}'.format(align))
+        if 'color' in node.rpr:
+            if node.rpr['color'] != '000000':
+                style.append('color: #{}'.format(node.rpr['color']))
 
-    if 'sz' in node.rpr:
-        size = int(node.rpr['sz']) / 2
+        if 'jc' in node.ppr:
+            # left right both
+            align = node.ppr['jc']
+            if align.lower() == 'both':
+                align = 'justify'
 
-        if ctx.options['scale_to_size']:
-            scale = int(round(size) / ctx.options['scale_to_size'] * 100.0)
-            style.append('font-size: {}%'.format(scale))
-        else:
-            style.append('font-size: {}pt'.format(size))
+            style.append('text-align: {}'.format(align))
 
-    if 'ind' in node.ppr:
-        if 'left' in node.ppr['ind']:
-            size = int(node.ppr['ind']['left']) / 10
-            style.append('margin-left: {}px'.format(size))
+        if 'ind' in node.ppr:
+            if 'left' in node.ppr['ind']:
+                size = int(node.ppr['ind']['left']) / 10
+                style.append('margin-left: {}px'.format(size))
 
-        if 'right' in node.ppr['ind']:
-            size = int(node.ppr['ind']['right']) / 10
-            style.append('margin-right: {}px'.format(size))
-
-        if 'first_line' in node.ppr['ind']:
-            size = int(node.ppr['ind']['first_line']) / 10
-            style.append('text-indent: {}px'.format(size))
+            if 'right' in node.ppr['ind']:
+                size = int(node.ppr['ind']['right']) / 10
+                style.append('margin-right: {}px'.format(size))
 
+            if 'first_line' in node.ppr['ind']:
+                size = int(node.ppr['ind']['first_line']) / 10
+                style.append('text-indent: {}px'.format(size))
 
     if len(style) == 0:
         return ''
@@ -496,7 +517,10 @@ def get_css_classes(document, style):
     >>> get_css_classes(doc, st)
     'header1 normal'
     """
-    return ' '.join([st.lower() for st in get_all_styles(document, style)[-1:]])
+    lst = [st.lower() for st in get_all_styles(document, style)[-1:]] + \
+        ['{}-fontsize'.format(st.lower()) for st in get_all_styles(document, style)[-1:]]
+
+    return ' '.join(lst)
 
 
 def serialize_paragraph(ctx, document, par, root, embed=True):
@@ -568,13 +592,21 @@ def serialize_paragraph(ctx, document, par, root, embed=True):
                 new_element, _element = _add_formatting('u', new_element, _element)
                 
                 _element.text = el.value()
+
+                for comment_id in ctx.opened_comments:
+                    document.comments[comment_id].text += ' ' + el.value()
             else:
                 new_element = etree.Element('span')
                 new_element.text = el.value()
 
+                for comment_id in ctx.opened_comments:
+                    document.comments[comment_id].text += ' ' + el.value()
+
+
             if ctx.options['embed_styles']:
-                if _text_style != '':
+                if _text_style != '' and _style != _text_style:
                     new_element.set('style', _text_style)
+
             # This is for situations when style has options and
             # text is trying to unset them
             # else:            
@@ -585,22 +617,27 @@ def serialize_paragraph(ctx, document, par, root, embed=True):
 
             if len(children) > 0:
                 _child_style = children[-1].get('style') or ''
-                
-                if new_element.tag == children[-1].tag and _text_style == _child_style and children[-1].tail is None:
+
+                if new_element.tag == children[-1].tag and (_text_style == _child_style or _child_style == '') and children[-1].tail is None:
                     txt = children[-1].text or ''
                     txt2 = new_element.text or ''
                     children[-1].text = u'{}{}'.format(txt, txt2)  
                     was_inserted = True
 
-                if _style == '' and _text_style == '' and new_element.tag == 'span':
-                    _e = children[-1]
+                if not was_inserted:
+#                    if _style == '' and _text_style == '' and new_element.tag == 'span':
+                    if _style == _text_style  and new_element.tag == 'span':
 
-                    txt = _e.tail or ''
-                    _e.tail = u'{}{}'.format(txt, new_element.text)
-                    was_inserted = True
+                        _e = children[-1]
+
+                        txt = _e.tail or ''
+                        _e.tail = u'{}{}'.format(txt, new_element.text)
+                        was_inserted = True
 
             if not was_inserted:
-                if _style == '' and _text_style == '' and new_element.tag == 'span':
+#                if _style == '' and _text_style == '' and new_element.tag == 'span':
+                if _style ==  _text_style  and new_element.tag == 'span':
+
                     txt = elem.text or ''
                     elem.text = u'{}{}'.format(txt, new_element.text)
                 else:
@@ -693,6 +730,30 @@ def serialize_footnote(ctx, document, el, root):
     return root
 
 
+def serialize_comment(ctx, document, el, root):
+    "Serializes comment."
+
+    # Check if option is turned on
+
+    if el.comment_type == 'end':
+        ctx.opened_comments.remove(el.cid)
+    else:
+        if el.comment_type != 'reference':
+            ctx.opened_comments.append(el.cid)
+
+        if ctx.options['comment_span']:
+            link = etree.SubElement(root, 'a')
+            link.set('href', '#')
+            link.set('class', 'comment-link')    
+            link.set('id', 'comment-id-' + el.cid)    
+
+            link.text = ''
+
+            fire_hooks(ctx, document, el, link, ctx.get_hook('comment'))
+
+    return root
+
+
 def serialize_endnote(ctx, document, el, root):
     "Serializes endnotes."
 
@@ -714,10 +775,47 @@ def serialize_endnote(ctx, document, el, root):
     return root
 
 
+def serialize_smarttag(ctx, document, el, root):
+    "Serializes smarttag."
+
+    if ctx.options['smarttag_span']:
+        _span = etree.SubElement(root, 'span', {'class': 'smarttag', 'data-smarttag-element': el.element})
+    else:
+        _span = root
+
+    for elem in el.elements:
+        _ser = ctx.get_serializer(elem)
+
+        if _ser:
+            _td = _ser(ctx, document, elem, _span)
+        else:
+            if isinstance(elem, doc.Text):
+                children = list(_span)
+
+                if len(children) == 0:
+                    _text = _span.text or u''
+
+                    _span.text = u'{}{}'.format(_text, elem.text)
+                else:
+                    _text = children[-1].tail or u''
+
+                    children[-1].tail = u'{}{}'.format(_text, elem.text)
+
+    fire_hooks(ctx, document, el, _span, ctx.get_hook('smarttag'))
+
+    return root
+
+
 def serialize_table(ctx, document, table, root):
     """Serializes table element.
     """
 
+    # What we should check really is why do we pass None as root element
+    # There is a good chance some content is missing after the import
+
+    if root is None:
+        return root
+
     if ctx.ilvl != None:
         root = close_list(ctx, root)
         ctx.ilvl, ctx.numid = None, None
@@ -923,16 +1021,22 @@ DEFAULT_OPTIONS = {
         doc.Math: serialize_math,
         doc.Break: serialize_break,
         doc.TextBox: serialize_textbox,
+        doc.Comment: serialize_comment,
         doc.Footnote: serialize_footnote,
         doc.Endnote: serialize_endnote,
-        doc.Symbol: serialize_symbol
+        doc.Symbol: serialize_symbol,
+        doc.SmartTag: serialize_smarttag
     },
 
     'hooks': {},
     'header': HeaderContext,
     'scale_to_size': None,
     'empty_paragraph_as_nbsp': False,
-    'embed_styles': True
+    'embed_styles': True,
+    'embed_fontsize': True,
+    'smarttag_span': False,
+    'comment_span': False,
+    'pretty_print': True
 }
 
 
@@ -957,25 +1061,11 @@ class Context:
         self.options = dict(DEFAULT_OPTIONS)
 
         if options:
-            if 'serializers' in options:
-                self.options['serializers'].update(options['serializers'])
-
-            if 'hooks' in options:
-                self.options['hooks'].update(options['hooks'])
-
-            # this is not that good way of updating options
-
-            if 'header' in options:
-                self.options['header'] = options['header']
-
-            if 'scale_to_size' in options:
-                self.options['scale_to_size'] = options['scale_to_size']
-
-            if 'empty_paragraph_as_nbsp' in options:
-                self.options['empty_paragraph_as_nbsp'] = options['empty_paragraph_as_nbsp']
-
-            if 'embed_styles' in options:
-                self.options['embed_styles'] = options['embed_styles']
+            for opt_key, opt_value in six.iteritems(options):
+                if type(opt_value) == type({}):
+                    self.options[opt_key].update(opt_value)
+                else:
+                    self.options[opt_key] = opt_value
 
         self.reset()
         self.header.init(document)
@@ -1013,6 +1103,7 @@ class Context:
         self.ilvl = None
         self.numid = None
 
+        self.opened_comments = []
         self.footnote_id = 0
         self.footnote_list = {}
         self.endnote_id = 0
@@ -1091,10 +1182,12 @@ def serialize_styles(document, prefix='', options=None):
             else:
                 break
 
-        content = "\n".join([get_style_css(ctx, st, embed=False) for st in styles])
-
+        content = "\n".join([get_style_css(ctx, st, embed=False, fontsize=1) for st in styles])
         css_content += "{0} .{1} {{ {2} }}\n\n".format(prefix, style_id.lower(), content)
 
+        content = "\n".join([get_style_css(ctx, st, embed=False, fontsize=2) for st in styles])
+        css_content += "{0} .{1}-fontsize {{ {2} }}\n\n".format(prefix, style_id.lower(), content)
+
     return css_content
 
 # Serialize list of elements into HTML
@@ -1123,7 +1216,7 @@ def serialize_elements(document, elements, options=None):
     # TODO:
     # - create footnotes now
 
-    return etree.tostring(tree_root, pretty_print=True, encoding="utf-8", xml_declaration=False)
+    return etree.tostring(tree_root, pretty_print=ctx.options.get('pretty_print', True), encoding="utf-8", xml_declaration=False)
 
 
 def serialize(document, options=None):
